package com.jmu.shiro_demo.controller;

import com.jmu.shiro_demo.service.UserService;
import com.jmu.shiro_demo.utils.CpachaUtil;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.apache.shiro.authz.annotation.RequiresRoles;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.OutputStream;

@Controller
public class RouteController {

    @Autowired
    private UserService userService;

    @GetMapping("/index")
    public String index() {
        return "index";
    }

    @GetMapping("/toLogin")
    public String toLogin() {
        return "login";
    }

    @GetMapping("/toUserAdd")
    @ResponseBody
    @RequiresPermissions("user:add")
    public String toUserAdd() {
        return "user/add";
    }

    @GetMapping("/toUserUpdate")
    @ResponseBody
    @RequiresPermissions("user:update")
    public String toUserUpdate() {
        return "user/update";
    }

    @GetMapping("/toUserDelete")
    @ResponseBody
    @RequiresPermissions("user:delete")
    public String toUserDelete() {
        return "user/delete";
    }

    @GetMapping("/unauthorized")
    public String toUnauthorized() {
        return "unauthorized";
    }

    @GetMapping("/logout")
    public String logout(){
        return userService.logout();
    }

    @GetMapping("/getAuthCode")
    public void getAuthCode(HttpSession session, HttpServletResponse response){
        CpachaUtil cpachaUtil = new CpachaUtil();
        String code = cpachaUtil.generatorVCode();
        session.setAttribute("code",code);
        BufferedImage bufferedImage = cpachaUtil.generatorVCodeImage(code, true);
        response.setContentType("image/png");
        try {
            OutputStream outputStream = response.getOutputStream();
            ImageIO.write(bufferedImage,"png",outputStream);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
